/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package j2objc.java.lang.ref;

import com.google.j2objc.annotations.AutoreleasePool;

import java.lang.ref.PhantomReference;
import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import junit.framework.TestCase;

import org.junit.Test;

import static org.junit.Assert.*;

/**
 * iOS-specific unit test for {@link PhantomReference}. The Android and Apache Harmony tests can't
 * be reused because they assume GC behavior. This class uses autorelease pools to manage when
 * references have phantom referents.
 *
 * @author Tom Ball
 */
public class PhantomReferenceTest extends TestCase {
  private PhantomReference<?> phantomRef;

  @Test
  public void testQueuedPhantomReference() {
    ReferenceQueue<? super Object> queue = new ReferenceQueue<Object>();
    for (@AutoreleasePool int i = 0; i < 1; i++) {
      Object referent = new Object();
      phantomRef = new PhantomReference<Object>(referent, queue);
      assertNull("phantomRef returned referent", phantomRef.get());
    }

    // Verify phantom reference was queued.
    Reference<?> queuedRef = queue.poll();
    assertNotNull("phantomRef wasn't queued", queuedRef);
  }
}
